<?php
namespace Teknisa\FAI\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

use Teknisa\Libs\Util\NameProviderImpl;
use Teknisa\Libs\Util\SMTP;

use Teknisa\FAI\Services\SMTPUnit;

class Fai01200ConhecimentoFornecimento extends \Zeedhi\Framework\Controller\Report {

    protected $newCode;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $environment;
    protected $procedureRelGesDireta;
    protected $dataSourceDirectory;
    const GROUP_RANGE = 11;

    public function __construct(ReportService $reportService, $newCode, $qr2Strategy, 
                                $birtStrategy, $environment, $connection, 
                                StoredProcedure $procedureRelGesDireta
    ) {
        $this->reportService = $reportService;
        $this->newCode       = $newCode;
        $this->qr2Strategy   = $qr2Strategy;
        $this->birtStrategy   = $birtStrategy;
        $this->environment   = $environment;
        $this->connection    = $connection;
        $this->procedureRelGesDireta = $procedureRelGesDireta;
        $this->dataSourceDirectory = InstanceManager::getInstance()->getParameter('dataSourceDirectory');
    }

    protected $reportMapping = array (
        'FAI01200' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'TCDCLIENTE'  => 'P_TCDCLIENTE',
                'DATAINI'     => 'P_DATAINI',
                'DATAFIN'     => 'P_DATAFIN',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'DTCDRS'      => 'P_DTCDRS',
                'NRORG'       => 'P_NRORG'
            ),
        ),
        'FAI01201' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'NRORG'       => 'P_NRORG'
            ),
        ),
        'FAI01202' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'NRORG'       => 'P_NRORG'

            ),
        ),
        'FAI01203' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTCDRS'      => 'P_DTCDRS',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'NRORG'       => 'P_NRORG'
            ),
        ),
        'FAI01208' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'CDGRUPSERV'  => 'P_CDGRUPSERV',
                'NRORG'       => 'P_NRORG'

            ),
        ),
        'FAI01209' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'NRORG'       => 'P_NRORG',
                'DTCDRS'      => 'P_DTCDRS'
            ),
        ),
        'FAI01210' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'DTCDRS'      => 'P_DTCDRS',
                'NRORG'       => 'P_NRORG'
            ),
        ),
        'FAI01211' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'    => 'P_CDFILIAL',
                'NRAPURACGD'  => 'P_NRAPURACGD',
                'CDCLIENTE'   => 'P_CDCLIENTE',
                'DTINICIAL'   => 'P_DTINICIAL',
                'DTFINAL'     => 'P_DTFINAL',
                'DTCDRS'      => 'P_DTCDRS',
                'NRORG'       => 'P_NRORG'
            ),
        ),
    );

    private function isCentralUnit($cdfilial){
        $sql = <<<SQL
            SELECT 
                IDCENTRALFATU 
            FROM 
                FILIAL 
            WHERE 
                CDFILIAL = :P_CDFILIAL
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute(array(
            'P_CDFILIAL' => $cdfilial,
            'P_NRORG'    => $this->environment->getNrOrg()
        ));
        $result = $stmt->fetch();
        if(!empty($result) && is_array($result) && isset($result['IDCENTRALFATU'])){
            return $result['IDCENTRALFATU'] == 'S';
        }
        return false;
    }

    // FAI01201 E FAI01202 são .CDS
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $row['DATAINI']    = $row['DTINICIAL'];
        $row['DATAFIN']    = $row['DTFINAL'];
        $row['NRORG']      = $this->environment->getNrOrg();
        $row['NRAPURACGD'] = $this->prepareDataReport($row['__report_name'], $row);
    }

    private function handleCentralParams( &$row ){
        $CENTRAL = $row['CDFILIALCENTRAL'] ?? null;
        if($CENTRAL) {
            $row['CDFILIAL'] = $CENTRAL;
            $row['AGGREGATE_UNIT'] = $row['ALL_AGGREGATE_UNITS'] === 'T' 
                ? $row['ALL_AGGREGATE_UNITS'] 
                : $this->criaListaValores(implode('_', $row['AGGREGATE_UNIT']));
        } else {
            $row['AGGREGATE_UNIT'] = null;
        }
    }

    /**
     * @return string NRAPURACGD
     */
    private function prepareDataReport($reportCode, $row){
        $this->handleCentralParams( $row );
        $row['NRAPURACGD'] = $this->callProcedureRelGesDireta($row);
        $this->treatCDRS($row['__report_name'], $row);
        if($this->isRequiredTrabRelatCfrForReport($reportCode)){
            $this->insertTrabRelatCfr( $row );
        }
        return $row['NRAPURACGD'];
    }

    public function generateDataReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $NRAPURACGD = $this->prepareDataReport($row['__report_name'], $row);
        $response->addDataSet(new DataSet("result", array('NRAPURACGD' => $NRAPURACGD)));
    }

    private function isRequiredCdrsForReport( $reportCode ){
        return in_array($reportCode, [ "FAI01200", "FAI01203", "FAI01209", "FAI01210", "FAI01211" ]);
    }

    private function isRequiredTrabRelatCfrForReport( $reportCode ){
        return in_array($reportCode, [ "FAI01201", "FAI01203" ]);
    }

    private function getAggregateUnits( $cdfilialcentral, $nrapuracgd ){
        $sql = <<<SQL
            SELECT DISTINCT
                CDFILIALAPGD AS CDFILIAL
            FROM
                TRAB_APURAC_GD TA
            WHERE
                TA.CDFILIALCENTRAL = :P_CDFILIAL AND
                TA.NRAPURACENTRAL = :P_NRAPURACGD
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'P_CDFILIAL' => $cdfilialcentral,
            'P_NRAPURACGD' => $nrapuracgd
        ]);
        $results = $stmt->fetchAll();
        if($results !== null && !empty($results)){
            $results = array_column($results, 'CDFILIAL');
        }
        return $results;
    }

    /**
     * @param $reportCode = "FAI01200"
     * @param $row = [
     *      "CDFILIAL" => "0001",
     *      "DTCDRS" => "01/01/2021",
     *      "QUINZENA" => "1" OR "2"
     * ]
     */
    private function treatCDRS( $reportCode, $row){
        if( $this->isRequiredCdrsForReport($reportCode) ) {
            $aggregateUnits = $this->getAggregateUnits( $row['CDFILIAL'], $row['NRAPURACGD'] ) ?? [];
            $params = clone $row;
            foreach( $aggregateUnits as $cdfilial ){
                $params['CDFILIAL'] = $cdfilial;
                $seq = $this->retornaSequencial($params);
                if($seq == null) {
                    $seqNextCode = $this->newCode->getCodeByTableName('NUMECDRS', $this->environment->getNrorg());
                    $this->insereCDRS($params, $seqNextCode);
                }
            }
        }
    }

    /**
     * @return string NRAPURACGD
     */
    private function callProcedureRelGesDireta($params){
        $CDCENTRALFATURAMENTO = $params['CDFILIALCENTRAL'] ?? null;
        $params['IDCENTRAL'] = $CDCENTRALFATURAMENTO ? 'S' : 'N';
        $NRORG = $this->environment->getNrorg();
        $aggregateUnits = $params['AGGREGATE_UNIT'] ?? 'T';
        $params['NRAPURACGD'] = $this->newCode->getCodeByTableName('APURACFATUR'.$params['CDFILIAL'], $NRORG);
        
        $this->procedureRelGesDireta->addParam($this->createParam('P_CDFILIAL',     $params['CDFILIAL']));
        $this->procedureRelGesDireta->addParam($this->createParam('P_DATAINI',      $params['DTINICIAL']));
        $this->procedureRelGesDireta->addParam($this->createParam('P_DATAFIN',      $params['DTFINAL']));
        $this->procedureRelGesDireta->addParam($this->createParam('P_NRAPURACGD',   $params['NRAPURACGD']));
        $this->procedureRelGesDireta->addParam($this->createParam('P_CDCLIENTE',    null,  Param::PARAM_TYPE_NULL));
        $this->procedureRelGesDireta->addParam($this->createParam('P_EQUIVALENCIA', 'N'));
        $this->procedureRelGesDireta->addParam($this->createParam('P_CENTRAL',      $params['IDCENTRAL']));
        $this->procedureRelGesDireta->addParam($this->createParam('P_NRLISTA_AGREGADAS',      $aggregateUnits));
        $this->procedureRelGesDireta->execute();
        return $params['NRAPURACGD'];
    }

    private function createParam($name, $value = null, $type = Param::PARAM_TYPE_STR, $inOut = Param::PARAM_INPUT, $length = null){
        return new Param($name, $inOut, $value , $type , $length );
    }

    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $row = $request->getRow();
        $reportCode = $row['__report_name'];
        $strategy = $this->reportMapping[$reportCode]['strategy'];
        if($strategy === 'QR2'){
            $this->deleteTrabApuracGD($row['NRAPURACGD'], $row['CDFILIAL']);
        }
    }

    private function deleteTrabApuracGD($nrapuracentral, $cdfilial){
        $sql = <<<SQL
        DELETE FROM TRAB_APURAC_GD
                WHERE 
                    CDFILIALCENTRAL = :P_CDFILIALCENTRAL OR
                    NRAPURACENTRAL  = :P_NRAPURACENTRAL
SQL;
        $params = array(
            'P_CDFILIALCENTRAL' => $cdfilial,
            'P_NRAPURACENTRAL'  => $nrapuracentral
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
    }

    private function deleteTrabRelatCfr($nrapuracentral, $cdfilial){
        $sql = <<<SQL
            DELETE FROM 
                TRABRELATCFR T
            WHERE 
                EXISTS (
                    SELECT G.CDFILIALAPGD
                    FROM 
                        TRAB_APURAC_GD G
                    WHERE 
                        T.CDFILIAL = G.CDFILIALAPGD AND
                        T.CDCLIENTE = G.CDCLIENTEAPGD AND
                        T.NRAPURACFAT = G.NRAPURACGD AND
                        G.CDFILIALCENTRAL = :P_CDFILIAL AND
                        G.NRAPURACENTRAL = :P_NRAPURACGD
                )
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'P_NRAPURACGD' => $nrapuracentral,
            'P_CDFILIAL' => $cdfilial
        ]);
    }

    public function retornaSequencial($row) {
        $sql = <<<SQL
        SELECT 
            NRCDRS
        FROM RELAT_CDRS
        WHERE 
            CDFILIAL = :P_CDFILIAL AND
            DTCDRS   = :P_DTCDRS
SQL;
        $params = array(
            'P_CDFILIAL' => $row['CDFILIAL'],
            'P_DTCDRS'   => $row['DTCDRS']
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $dataSet = $query->fetchAll();
        return $dataSet;
    }

    public function insereCDRS($row, $seq) {
        $sql = <<<SQL
        INSERT INTO RELAT_CDRS
            (
                CDFILIAL, CDCLIENTE,  NRCDRS,    CDSERVFILI, 
                DTCDRS,   NRQUINZENA, CDPRODUTO, VRPRECUNIT
            )
            (
                SELECT DISTINCT 
                    TA.CDFILIALAPGD, TA.CDCLIENTEAPGD, :P_NRCDRS,        TA.CDSERVFILIAPGD, 
                    :P_DTCDRS,      :P_NRQUINZENA,     TA.CDPRODUTOAPGD, 0
                FROM TRAB_APURAC_GD TA
                    WHERE ( TA.CDFILIALAPGD = :P_CDFILIAL ) AND
                          ( TA.NRAPURACENTRAL = :P_NRAPURACGD ) AND
                          ( VALOR_NA_LISTA(TA.CDCLIENTEAPGD, :P_CDCLIENTEAPGD) <> 0)
            )
SQL;
        $params = array(
            'P_NRCDRS'        => $seq,
            'P_DTCDRS'        => $row['DTCDRS'],
            'P_NRQUINZENA'    => $row['QUINZENA'],
            'P_CDFILIAL'      => $row['CDFILIAL'],
            'P_NRAPURACGD'    => $row['NRAPURACGD'],
            'P_CDCLIENTEAPGD' => 'T'
        );
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

    public function sendToClientEmail(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $subject = $row['TITLE'];
        $message = $row['BODY'];
        $row['NRORG'] = $this->environment->getNrOrg();
        $row['DATAINI'] = $row['DTINICIAL'];
        $row['DATAFIN'] = $row['DTFINAL'];
        $clients = $this->findClientsToSendEmail($row);
        $reportName = parent::getReportName($row);
        $reportStrategy = parent::getReportStrategy($reportName);
        $smtp = new SMTPUnit($this->connection, $this->environment);
        $smtpParams = $smtp->findSmtpParamsUnit($row['CDFILIAL']);
        
        $smtp->setSmtpParams($smtpParams);
        $smtp->setSubject($subject);
            
        foreach( $clients as $client ) {
            $row['CDCLIENTE'] = $client['CDCLIENTE'];
            $parametersRow = parent::prepareParametersRow($reportName, $row);
            $parametersRow['P_ENVIAEMAIL'] = 'S';
            $url = $this->reportService->createRemoteReport($reportStrategy, $reportName, $parametersRow, true);
            $smtp->addReportToAttachment($url, $client['CDCLIENTE'], 'pdf'); 
            
            $dsEmail = $this->isValidEmail($client['DSEMAIL'], $client["NMRAZSOCCLIE"]);
            if(!empty($dsEmail['INVALID_EMAILS'])) {
                return $response->addDataSet(new DataSet("invalidEmails", $dsEmail['INVALID_EMAILS']));
            }
            
            $smtp->sendMail($message, $dsEmail);
            $smtp->deleteTemporaryFiles();
        }
        $this->deleteTrabRelatCfr($row['NRAPURACGD'], $row['CDFILIAL']);
        $this->deleteTrabApuracGD($row['NRAPURACGD'], $row['CDFILIAL']);
    }
    
    public function isValidEmail($dsEmail, $nmRazSoClie) {
        $replaceDsEmail = str_replace(",", ";", $dsEmail);
        $replaceDsEmail = str_replace(" ", "", $replaceDsEmail);
        $emailValidationRegex = "/^\w+([\.-]?\w+)@\w+([\.-]?\w+)(\.\w{2,3})+$/";
        $invalidEmailmatches = array(
            "INVALID_EMAILS" => array()   
        );
        $dsEmail = strpos($replaceDsEmail, ";") ? explode(";", $replaceDsEmail) : $replaceDsEmail;
        if(is_array($dsEmail)) {
            foreach($dsEmail as $email) {
                if(!empty($email)) {
                    $verifyEmail = preg_match($emailValidationRegex, $email);
                }
                if($verifyEmail == 0) {
                    array_push($invalidEmailmatches['INVALID_EMAILS'], (object)["CLIENTE" => $nmRazSoClie, "EMAIL" =>$email]);
                }
            }
        } else {
            $verifyEmail = preg_match($emailValidationRegex, $dsEmail);
            if($verifyEmail == 0) {
                array_push($invalidEmailmatches['INVALID_EMAILS'], (object)["CLIENTE" => $nmRazSoClie, "EMAIL" => $dsEmail]);
            }
        }
        if(!empty($invalidEmailmatches['INVALID_EMAILS'])){
            return $invalidEmailmatches;
        } else {
            return $dsEmail;
        }    
    }

    public function closeWindowCFR(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        if(isset($row['NRAPURACGD']) && isset($row['CDFILIAL'])) {
            $this->deleteTrabRelatCfr($row['NRAPURACGD'], $row['CDFILIAL']);
            $this->deleteTrabApuracGD($row['NRAPURACGD'], $row['CDFILIAL']);
        }
    }

    private function findClientsToSendEmail($row){
        $dataSourceName = 'cliente_email_trab_apurac';
        $nameProvider = new NameProviderImpl($this->dataSourceDirectory); 
        $dataSource = $nameProvider->getDataSourceByName($dataSourceName);
        $cliente = $row['CDCLIENTE'];
        if($row['TCDCLIENTE'] == 'T') {
            $cliente = 'T';
        }
        $cliente = $this->criaListaValores($cliente);
        $sql = $dataSource->getQuery();
        $NRAPURACGD = $row['NRAPURACGD'] ?? null; 
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'P_CDFILIAL'   => $row['CDFILIAL'],
            'P_NRAPURACGD' => $row['NRAPURACGD'],
            'P_DTINICIAL'  => $row['DTINICIAL'],
            'P_DTFINAL'    => $row['DTFINAL'],
            'P_CDCLIENTE'  => $cliente,
            'P_CDGRUPSERV' => empty($row['CDGRUPSERV']) ? 'T' : $row['CDGRUPSERV']
        ]);
        return $stmt->fetchAll();
    }

    private function findClientsPeriod( $row ){
        $sql = <<<SQL
            SELECT
                DCS.CDFILIALAPGD,
                DCS.CDCLIENTEAPGD,
                DCS.CDSERVFILIAPGD,
                DCS.DTDATA AS DTAPURACFATGD,
                NVL(TA.NREFETIVOSAPGD, 0) AS NREFETIVOSAPGD
            FROM
                TRAB_APURAC_GD TA,
                (
                    SELECT
                        CDFILIALAPGD,
                        CDFILIALCENTRAL,
                        NRAPURACGD,
                        NRAPURACENTRAL,
                        DTDATA,
                        CDCLIENTEAPGD,
                        CDSERVFILIAPGD
                    FROM
                        DATA,
                        (
                            SELECT DISTINCT
                                CDFILIALAPGD,
                                CDFILIALCENTRAL,
                                NRAPURACENTRAL,
                                NRAPURACGD,
                                CDCLIENTEAPGD,
                                CDSERVFILIAPGD
                            FROM
                                TRAB_APURAC_GD
                            WHERE
                                ( CDFILIALCENTRAL = :P_CDFILIALAPGD )
                                AND ( NRAPURACENTRAL = :P_NRAPURACGD )
                                AND ( VALOR_NA_LISTA(CDCLIENTEAPGD, :P_CDCLIENTEAPGD) <> 0 )
                        ) CS
                    WHERE
                        DTDATA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                ) DCS
            WHERE
                ( DCS.CDFILIALCENTRAL = :P_CDFILIALAPGD )
                AND ( DCS.NRAPURACENTRAL = :P_NRAPURACGD)
                AND ( DCS.CDFILIALAPGD = TA.CDFILIALAPGD (+) )
                AND ( DCS.NRAPURACGD = TA.NRAPURACGD (+) )
                AND ( DCS.CDCLIENTEAPGD = TA.CDCLIENTEAPGD (+) )
                AND ( DCS.CDSERVFILIAPGD = TA.CDSERVFILIAPGD (+) )
                AND ( DCS.DTDATA = TA.DTAPURACFATGD (+) )
            ORDER BY
                DCS.CDFILIALAPGD,
                DCS.CDCLIENTEAPGD,
                DCS.CDSERVFILIAPGD,
                DCS.DTDATA
SQL;
        $stmt = $this->connection->prepare($sql);
        $stmt->execute([
            'P_CDFILIALAPGD'  => $row['CDFILIAL'],
            'P_NRAPURACGD'    => $row['NRAPURACGD'],
            'P_CDCLIENTEAPGD' => $row['CDCLIENTE'],
            'P_DTINICIAL'     => $row['DTINICIAL'],
            'P_DTFINAL'       => $row['DTFINAL']
        ]);
        return $stmt->fetchAll();
    }

    private function getValuesConcatInRow( $row, $arrayKeys ){
        $keysWithValueNull = array_fill_keys($arrayKeys, null);
        $value = implode(
            '',
            array_intersect_key($row, $keysWithValueNull)
        );
        return $value;
    }
    
    private function getDefaultValueParams($sql, $defaultValue = null){
        preg_match_all("/(?<=:)\w+/", $sql, $match);
        $firstGroupMatch = $match[0];
        if(!empty($firstGroupMatch)){
            return array_fill_keys($firstGroupMatch, $defaultValue);
        }
        return [];
    }

    private function insertTrabRelatCfr( $row ){
        $sql = <<<SQL
            INSERT INTO TRABRELATCFR (
                CDFILIAL,
                CDCLIENTE,
                NRAPURACFAT,
                CDSERVFILI,
                DTRELATCFR01, DTRELATCFR02, DTRELATCFR03, DTRELATCFR04, DTRELATCFR05, DTRELATCFR06, 
                DTRELATCFR07, DTRELATCFR08, DTRELATCFR09, DTRELATCFR10, DTRELATCFR11,
                QTEFETIVOS01, QTEFETIVOS02, QTEFETIVOS03, QTEFETIVOS04, QTEFETIVOS05, QTEFETIVOS06,
                QTEFETIVOS07, QTEFETIVOS08, QTEFETIVOS09, QTEFETIVOS10, QTEFETIVOS11,
                NRSEQRELAT
            ) VALUES (
                :P_CDFILIAL,
                :P_CDCLIENTE,
                :P_NRAPURACFAT,
                :P_CDSERVFILI,
                :P_DTRELATCFR1, :P_DTRELATCFR2, :P_DTRELATCFR3, :P_DTRELATCFR4,  :P_DTRELATCFR5, :P_DTRELATCFR6,
                :P_DTRELATCFR7, :P_DTRELATCFR8, :P_DTRELATCFR9, :P_DTRELATCFR10, :P_DTRELATCFR11,
                :P_QTEFETIVOS1, :P_QTEFETIVOS2, :P_QTEFETIVOS3, :P_QTEFETIVOS4,  :P_QTEFETIVOS5, :P_QTEFETIVOS6,
                :P_QTEFETIVOS7, :P_QTEFETIVOS8, :P_QTEFETIVOS9, :P_QTEFETIVOS10, :P_QTEFETIVOS11,
                :P_NRSEQRELAT
            )
SQL;
        $records = $this->findClientsPeriod( $row );
        $this->connection->beginTransaction();
        try {
            $stmt = $this->connection->prepare($sql);
            $initParams = [
                'P_NRAPURACFAT' => $row['NRAPURACGD']
            ];
            $params = $initParams;
            $PK = ['CDFILIALAPGD', 'CDCLIENTEAPGD', 'CDSERVFILIAPGD'];
            $lastPkValue = '';
            $countInsert = 1;
            $indexParam = 1;
            foreach( $records as $index => $client ){
                $currentPkValue = $this->getValuesConcatInRow($client, $PK);
                $afterPkValue = $index < count($records)-1 ? $this->getValuesConcatInRow($records[$index+1], $PK) : null;
                $params["P_DTRELATCFR$indexParam"] = $client['DTAPURACFATGD'];
                $params["P_QTEFETIVOS$indexParam"] = $client['NREFETIVOSAPGD'];
                if($indexParam === self::GROUP_RANGE || $afterPkValue != $currentPkValue){
                    $params['P_NRSEQRELAT'] = $countInsert;
                    $params['P_CDFILIAL'] = $client['CDFILIALAPGD'];
                    $params['P_CDCLIENTE'] = $client['CDCLIENTEAPGD'];
                    $params['P_CDSERVFILI'] = $client['CDSERVFILIAPGD'];
                    $newParams = array_merge(
                        $this->getDefaultValueParams($sql),
                        $params
                    );
                    $stmt->execute($newParams);
                    $params = $initParams;
                    $countInsert++;
                    $indexParam = 1;
                } else {
                    $indexParam++;
                }
                
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function criaListaValores($valores) {
        if(strlen($valores) <= 4000) {
            return $valores;
        } else {
            $lista = explode('_', $valores);
            $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrgTrab(), 1, 10, false);
            try{
                foreach($lista as $key) {
                    $this->inserinsereLista($nrlista, $key);
                }
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e) {
                throw $e;
            }
        }
    }
    
    public function insereLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA"   => $nrlista,
            "CDLISTA"   => $cdlista,
            "NRORG"     => $this->environment->getNrOrgTrab()
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA, NRORG) VALUES (:NRLISTA, :CDLISTA, :NRORG)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
}